@@ -14,12 +14,14 @@ use std::net::{
     IpAddr, Ipv4Addr, Ipv6Addr,
     SocketAddr, SocketAddrV4, SocketAddrV6,
 };
+use std::num::Wrapping;
 use std::ops::{Range, RangeFrom, RangeTo, RangeFull};
 use std::path::PathBuf;
 use std::sync::Arc;
-use std::time::Duration;
+use std::time::{UNIX_EPOCH, Duration, SystemTime};
 
 use rand::{self, Rng, RngCore};
+use rand::seq::SliceRandom;
 
 /// `Gen` wraps a `rand::RngCore` with parameters to control the distribution of
 /// random values.
@@ -194,7 +196,7 @@ impl<A: Arbitrary, B: Arbitrary> Arbitrary for Result<A, B> {
 }
 
 macro_rules! impl_arb_for_single_tuple {
-    ($(($type_param:ident, $tuple_index:tt, $cloned_for_index:ident),)*) => {
+    ($(($type_param:ident, $tuple_index:tt),)*) => {
         impl<$($type_param),*> Arbitrary for ($($type_param,)*)
             where $($type_param: Arbitrary,)*
         {
@@ -207,88 +209,41 @@ macro_rules! impl_arb_for_single_tuple {
             }
 
             fn shrink(&self) -> Box<Iterator<Item=($($type_param,)*)>> {
+                let iter = ::std::iter::empty();
                 $(
-                    let $cloned_for_index = self.clone();
+                    let cloned = self.clone();
+                    let iter = iter.chain(self.$tuple_index.shrink().map(move |shr_value| {
+                        let mut result = cloned.clone();
+                        result.$tuple_index = shr_value;
+                        result
+                    }));
                 )*
-
-                Box::new(
-                    ::std::iter::empty()
-                    $(
-                        .chain(self.$tuple_index.shrink().map(move |shr_value| {
-                            let mut result = $cloned_for_index.clone();
-                            result.$tuple_index = shr_value;
-                            result
-                        }))
-                    )*
-                )
+                Box::new(iter)
             }
         }
     };
 }
 
-macro_rules! impl_arb_for_single_tuple_with_reversed_args {
-    (@internal [$($output:tt,)*]) => {
-        impl_arb_for_single_tuple!($($output,)*);
-    };
-    (@internal [$($output:tt,)*] $first:tt, $($rest:tt,)*) => {
-        impl_arb_for_single_tuple_with_reversed_args!(@internal [$first, $($output,)*] $($rest,)*);
-    };
-    (($(($type_param:ident, $tuple_index:tt, $cloned_for_index:ident),)+),) => {
-        impl_arb_for_single_tuple_with_reversed_args!(@internal
-            []
-            $(($type_param, $tuple_index, $cloned_for_index),)+
-        );
-    };
-}
-
-macro_rules! impl_arb_for_tuples_with_reversed_args {
-    (@internal [$($output:tt,)*]) => {
-        $(
-            impl_arb_for_single_tuple_with_reversed_args!($output,);
-        )*
-    };
-    (@internal [$($output:tt,)*] $first:tt, $($rest:tt,)*) => {
-        impl_arb_for_tuples_with_reversed_args!(@internal
-            [($first, $($rest,)*), $($output,)*]
-            $($rest,)*
-        );
-    };
-    ($(($type_param:ident, $tuple_index:tt, $cloned_for_index:ident),)*) => {
-        impl_arb_for_tuples_with_reversed_args!(@internal
-            []
-            $(($type_param, $tuple_index, $cloned_for_index),)*
-        );
-    };
-}
-
 macro_rules! impl_arb_for_tuples {
-    (@internal [$($reversed_args:tt,)*]) => {
-        impl_arb_for_tuples_with_reversed_args!($($reversed_args,)*);
+    (@internal [$($acc:tt,)*]) => { };
+    (@internal [$($acc:tt,)*] ($type_param:ident, $tuple_index:tt), $($rest:tt,)*) => {
+        impl_arb_for_single_tuple!($($acc,)* ($type_param, $tuple_index),);
+        impl_arb_for_tuples!(@internal [$($acc,)* ($type_param, $tuple_index),] $($rest,)*);
     };
-    (@internal
-        [$($reversed_args:tt,)*]
-        ($type_param:ident, $tuple_index:tt, $cloned_for_index:ident),
-        $($rest:tt,)*
-    ) => {
-        impl_arb_for_tuples!(@internal
-            [($type_param, $tuple_index, $cloned_for_index), $($reversed_args,)*]
-            $($rest,)*
-        );
-    };
-    ($(($type_param:ident, $tuple_index:tt, $cloned_for_index:ident),)*) => {
-        impl_arb_for_tuples!(@internal [] $(($type_param, $tuple_index, $cloned_for_index),)*);
+    ($(($type_param:ident, $tuple_index:tt),)*) => {
+        impl_arb_for_tuples!(@internal [] $(($type_param, $tuple_index),)*);
     };
 }
 
 impl_arb_for_tuples! {
-    (A, 0, cloned_for_0),
-    (B, 1, cloned_for_1),
-    (C, 2, cloned_for_2),
-    (D, 3, cloned_for_3),
-    (E, 4, cloned_for_4),
-    (F, 5, cloned_for_5),
-    (G, 6, cloned_for_6),
-    (H, 7, cloned_for_7),
+    (A, 0),
+    (B, 1),
+    (C, 2),
+    (D, 3),
+    (E, 4),
+    (F, 5),
+    (G, 6),
+    (H, 7),
 }
 
 impl<A: Arbitrary> Arbitrary for Vec<A> {
@@ -555,7 +510,7 @@ impl Arbitrary for PathBuf {
             PathBuf::from("../../.."),
             PathBuf::new(),
         ];
-        let mut p = g.choose(choices).unwrap().clone();
+        let mut p = choices.choose(g).unwrap().clone();
         p.extend(Vec::<OsString>::arbitrary(g).iter());
         p
     }
@@ -633,7 +588,7 @@ impl Arbitrary for char {
             }
             60...84 => {
                 // Characters often used in programming languages
-                *g.choose(&[
+                [
                     ' ', ' ', ' ',
                     '\t',
                     '\n',
@@ -641,11 +596,11 @@ impl Arbitrary for char {
                     '_', '-', '=', '+','[', ']', '{', '}', ':', ';', '\'', '"',
                     '\\', '|',',','<','>','.','/','?',
                     '0', '1','2','3','4','5','6','7','8','9',
-                ]).unwrap()
+                ].choose(g).unwrap().to_owned()
             }
             85...89 => {
                 // Tricky Unicode, part 1
-                *g.choose(&[
+                [
                     '\u{0149}', // a deprecated character
                     '\u{fff0}', // some of "Other, format" category:
                     '\u{fff1}','\u{fff2}','\u{fff3}','\u{fff4}','\u{fff5}',
@@ -667,7 +622,7 @@ impl Arbitrary for char {
                     '\u{1680}',
                     // other space characters are already covered by two next
                     // branches
-                ]).unwrap()
+                ].choose(g).unwrap().to_owned()
             }
             90...94 => {
                 // Tricky unicode, part 2
@@ -748,7 +703,7 @@ macro_rules! unsigned_arbitrary {
 unsigned_arbitrary! {
     usize, u8, u16, u32, u64
 }
-#[cfg(feature = "i128")]
+
 unsigned_arbitrary! {
     u128
 }
@@ -821,7 +776,7 @@ macro_rules! signed_arbitrary {
 signed_arbitrary! {
     isize, i8, i16, i32, i64
 }
-#[cfg(feature = "i128")]
+
 signed_arbitrary! {
     i128
 }
@@ -848,6 +803,15 @@ impl Arbitrary for f64 {
     }
 }
 
+impl<T: Arbitrary> Arbitrary for Wrapping<T> {
+    fn arbitrary<G: Gen>(g: &mut G) -> Wrapping<T> {
+        Wrapping(T::arbitrary(g))
+    }
+    fn shrink(&self) -> Box<Iterator<Item=Wrapping<T>>> {
+        Box::new(self.0.shrink().map(|inner| Wrapping(inner)))
+    }
+}
+
 impl<T: Arbitrary + Clone + PartialOrd> Arbitrary for Range<T> {
     fn arbitrary<G: Gen>(g: &mut G) -> Range<T> {
         Arbitrary::arbitrary(g) .. Arbitrary::arbitrary(g)
@@ -916,31 +880,25 @@ impl<A: Arbitrary + Sync> Arbitrary for Arc<A> {
     }
 }
 
-#[cfg(feature = "unstable")]
-mod unstable_impls {
-    use {Arbitrary, Gen};
-    use std::time::{Duration, SystemTime, UNIX_EPOCH};
-
-    impl Arbitrary for SystemTime {
-        fn arbitrary<G: Gen>(gen: &mut G) -> Self {
-            let after_epoch = bool::arbitrary(gen);
-            let duration = Duration::arbitrary(gen);
-            if after_epoch {
-                UNIX_EPOCH + duration
-            } else {
-                UNIX_EPOCH - duration
-            }
+impl Arbitrary for SystemTime {
+    fn arbitrary<G: Gen>(gen: &mut G) -> Self {
+        let after_epoch = bool::arbitrary(gen);
+        let duration = Duration::arbitrary(gen);
+        if after_epoch {
+            UNIX_EPOCH + duration
+        } else {
+            UNIX_EPOCH - duration
         }
+    }
 
-        fn shrink(&self) -> Box<Iterator<Item=Self>> {
-            let duration = match self.duration_since(UNIX_EPOCH) {
-                Ok(duration) => duration,
-                Err(e) => e.duration(),
-            };
-            Box::new(duration.shrink().flat_map(|d| {
-                vec![UNIX_EPOCH + d, UNIX_EPOCH - d]
-            }))
-        }
+    fn shrink(&self) -> Box<Iterator<Item=Self>> {
+        let duration = match self.duration_since(UNIX_EPOCH) {
+            Ok(duration) => duration,
+            Err(e) => e.duration(),
+        };
+        Box::new(duration.shrink().flat_map(|d| {
+            vec![UNIX_EPOCH + d, UNIX_EPOCH - d]
+        }))
     }
 }
 
@@ -958,6 +916,7 @@ mod test {
     };
     use std::fmt::Debug;
     use std::hash::Hash;
+    use std::num::Wrapping;
     use std::path::PathBuf;
     use super::Arbitrary;
 
@@ -980,7 +939,7 @@ mod test {
         super::Arbitrary::arbitrary(&mut gen())
     }
 
-    fn gen() -> super::StdGen<rand::ThreadRng> {
+    fn gen() -> super::StdGen<rand::rngs::ThreadRng> {
         super::StdGen::new(rand::thread_rng(), 5)
     }
 
@@ -1077,7 +1036,6 @@ mod test {
         eq(0i64, vec![]);
     }
 
-    #[cfg(feature="i128")]
     #[test]
     fn ints128() {
         eq(5i128, vec![0, 3, 4]);
@@ -1115,7 +1073,6 @@ mod test {
         eq(0u64, vec![]);
     }
 
-    #[cfg(feature = "i128")]
     #[test]
     fn uints128() {
         eq(5u128, vec![0, 3, 4]);
@@ -1163,6 +1120,13 @@ mod test {
         eq(1.5, vec![0.0]);
     }
 
+    #[test]
+    fn wrapping_ints32() {
+        eq(Wrapping(5i32), vec![Wrapping(0), Wrapping(3), Wrapping(4)]);
+        eq(Wrapping(-5i32), vec![Wrapping(5), Wrapping(0), Wrapping(-3), Wrapping(-4)]);
+        eq(Wrapping(0i32), vec![]);
+    }
+
     #[test]
     fn vecs() {
         eq({let it: Vec<isize> = vec![]; it}, vec![]);
